/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.camel.quarkus.component.hazelcast.it.model;

import java.util.concurrent.TimeUnit;

public class HazelcastMapRequest {
    private String id;
    private Object value;
    private Long ttl;
    private TimeUnit ttlUnit;

    public HazelcastMapRequest() {
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public Object getValue() {
        return value;
    }

    public void setValue(Object value) {
        this.value = value;
    }

    public Long getTtl() {
        return ttl;
    }

    public void setTtl(Long ttl) {
        this.ttl = ttl;
    }

    public TimeUnit getTtlUnit() {
        return ttlUnit;
    }

    public void setTtlUnit(TimeUnit ttlUnit) {
        this.ttlUnit = ttlUnit;
    }

    public HazelcastMapRequest withId(String id) {
        setId(id);
        return this;
    }

    public HazelcastMapRequest withVaLue(Object value) {
        setValue(value);
        return this;
    }

    public HazelcastMapRequest withTtl(Long ttl, TimeUnit ttlUnit) {
        setTtl(ttl);
        setTtlUnit(ttlUnit);
        return this;
    }
}
